using System;
using System.Collections.Generic;
using System.Linq;
using Upsilon.BaseTypes.Number;
using Upsilon.BaseTypes.ScriptTypeInterfaces;
using Upsilon.Binder;
using Upsilon.Evaluator;
using Upsilon.Text;

namespace Upsilon.BaseTypes
{
    internal class ScriptTable : ScriptType, IIndexable, IScopeOwner, IIterable, ILengthType
    {
        public EvaluationScope EvaluationScope { get; }

        public ScriptTable(EvaluationScope scope)
        {
            EvaluationScope = scope;
        }

        public override Type Type => Type.Table;
        public override object ToCSharpObject()
        {
            return EvaluationScope.Variables.ToDictionary(x => x.Key, x => x.Value.ToCSharpObject());
        }

        public override System.Type GetCSharpType()
        {
            return typeof(Dictionary<string, object>);
        }

        public ScriptType Get(Diagnostics diagnostics, TextSpan span, ScriptType index, EvaluationScope scope)
        {
            var s = index.ToString();
            if (EvaluationScope.TryGet(s, out var o))
            {
                return o;
            }
            diagnostics.LogError("Cannot find member 's' on Table", span);
            return new ScriptNull();
        }

        public void Set(Diagnostics diagnostics, TextSpan span, ScriptType index, ScriptType value)
        {
            var s = index.ToString();
            EvaluationScope.CreateLocal(new VariableSymbol(s, value.Type, false), value);
        }

        public ScriptType GetValueFromIndex(ScriptType index)
        {
            if (EvaluationScope.TryGet(index.ToString(), out var result))
            {
                return result;
            }
            throw new Exception($"Can't find key '{index}' in table.");
        }

        public IEnumerator<ScriptType> GetEnumerator()
        {
            return Enumerator();
        }

        private IEnumerator<ScriptType> Enumerator()
        {
            foreach (var variable in EvaluationScope.Variables)
            {
                yield return variable.Key.ToScriptType();
            }
        }

        public ScriptNumberLong Length()
        {
            return new ScriptNumberLong(EvaluationScope.Variables.Count);
        }
    }
}