using System.Collections;
using Upsilon.BaseTypes.Number;
using Upsilon.BaseTypes.UserData;

namespace Upsilon.BaseTypes
{
    internal static class TypeConversion
    {
        public static ScriptType ToScriptType(this object o)
        {
            if (o is ScriptType type) return type;
            switch (o)
            {
                case bool b:
                    return new ScriptBoolean(b);
                case int i:
                    return new ScriptNumberLong(i);
                case long i:
                    return new ScriptNumberLong(i);
                case float f:
                    return new ScriptNumberDouble(f);
                case double f:
                    return new ScriptNumberDouble(f);
                case string s:
                    return new ScriptString(s);
                case null:
                    return new ScriptNull();
                default:
                    return ConvertUserData(o);
            }
        }

        private static ScriptType ConvertUserData(object o)
        {
            if (o is IList ls)
            {
                return new ListUserData(ls);
            }
            if (o is IDictionary dic)
            {
                return new DictionaryUserData(dic);
            }
            return new GenericUserData(o);
        }
    }
}