using System.Collections;
using System.Collections.Generic;
using Upsilon.BaseTypes.Number;
using Upsilon.BaseTypes.ScriptTypeInterfaces;
using Upsilon.Evaluator;
using Upsilon.Text;

namespace Upsilon.BaseTypes.UserData
{
    internal class DictionaryUserData : ScriptType, IUserData, ILengthType
    {
        public IDictionary Dictionary { get; }

        public DictionaryUserData(IDictionary dictionary)
        {
            Dictionary = dictionary;
        }

        public ScriptType Get(Diagnostics diagnostics, TextSpan span, ScriptType index, EvaluationScope scope)
        {
            var key = index.ToCSharpObject();
            if (Dictionary.Contains(key))
            {
                return Dictionary[key].ToScriptType();
            }
            //TODO: log error
            return new ScriptNull();
        }

        public void Set(Diagnostics diagnostics, TextSpan span, ScriptType index, ScriptType value)
        {
            var key = index.ToCSharpObject();
            if (Dictionary.Contains(key))
            {
                Dictionary[key] = value.ToCSharpObject();
            }
            else
            {
                Dictionary.Add(key, value.ToCSharpObject());
            }
        }

        public override Type Type => Type.UserData;
        public override object ToCSharpObject()
        {
            return Dictionary;
        }

        public override System.Type GetCSharpType()
        {
            return Dictionary.GetType();
        }

        public ScriptNumberLong Length()
        {
            return new ScriptNumberLong(Dictionary.Count);
        }
    }
}