using Upsilon.Evaluator;
using Upsilon.Text;

namespace Upsilon.BaseTypes.UserData
{
    internal class GenericUserData : ScriptType, IUserData
    {
        public GenericUserData(object dictionary)
        {
            Value = dictionary;
            _typeInfo = UserDataTypeHandler.GetTypeInfo(dictionary.GetType());
        }
        public override Type Type => Type.UserData;

        protected virtual object Value { get; }
        private readonly UserDataType _typeInfo;

        public override object ToCSharpObject()
        {
            return Value;
        }

        public override System.Type GetCSharpType()
        {
            return Value.GetType();
        }

        public virtual ScriptType Get(Diagnostics diagnostics, TextSpan span, ScriptType index, EvaluationScope scope)
        {
            var s = index.ToCSharpObject().ToString();
            var (type, failed, error) = _typeInfo.Get(Value, s);
            if (failed)
            {
                diagnostics.LogError(error, span);
            }
            return type;
        }

        public virtual void Set(Diagnostics diagnostics, TextSpan span, ScriptType index, ScriptType value)
        {
            var s = index.ToCSharpObject().ToString();
            var (failed, error) = _typeInfo.Set(Value, s, value);
            if (failed)
            {
                diagnostics.LogError(error, span);
            }
        }

        public (ScriptType Type, bool Failed) BinaryOperator(ScriptType par1, OperatorType op, ScriptType par2)
        {
            return _typeInfo.BinaryOperator(Value, par1, op, par2);
        }

        public (ScriptType Type, bool Failed) UnaryOperator(ScriptType par1, OperatorType op)
        {
            return _typeInfo.UnaryOperator(Value, par1, op);
        }
    }
}