using System;
using System.Collections;
using System.Collections.Generic;
using Upsilon.BaseTypes.Number;
using Upsilon.BaseTypes.ScriptTypeInterfaces;
using Upsilon.Evaluator;
using Upsilon.Text;

namespace Upsilon.BaseTypes.UserData
{
    internal class ListUserData : ScriptType, IUserData, IIterable, ILengthType
    {
        private IList List { get; }

        public ListUserData(IList list)
        {
            List = list;
        }

        public ScriptType Get(Diagnostics diagnostics, TextSpan span, ScriptType index, EvaluationScope scope)
        {
            int i;
            if (index.Type == Type.String)
            {
                var str = (ScriptString)index;
                i = int.Parse(str.Value) - 1;
            }
            else if (index.Type == Type.Number)
            {
                var ind = (Number.ScriptNumberLong) index;
                i = (int) ind.Value - 1;
            }
            else
            {
                return null;
            }
            return List[i].ToScriptType();
        }

        public void Set(Diagnostics diagnostics, TextSpan span, ScriptType index, ScriptType value)
        {
            throw new System.NotImplementedException();
        }

        public override Type Type => Type.UserData;
        public override object ToCSharpObject()
        {
            return List;
        }

        public override System.Type GetCSharpType()
        {
            return List.GetType();
        }

        public ScriptType GetValueFromIndex(ScriptType index)
        {
            var num = (ScriptNumberLong)index;
            return List[(int) num.Value].ToScriptType();
        }

        public IEnumerator<ScriptType> GetEnumerator()
        {
            for (int i = 0; i < List.Count; i++)
            {
                yield return new ScriptNumberLong(i);
            }
        }

        public ScriptNumberLong Length()
        {
            return new ScriptNumberLong(List.Count);
        }
    }
}