using System.Collections.Generic;
using System.Linq;
using System.Reflection;

namespace Upsilon.BaseTypes.UserData
{
    public class UserDataMethod
    {
        private class UserDataMethodPart
        {
            public UserDataMethodPart(MethodInfo method)
            {
                Method = method;
                Parameters = method.GetParameters().Select(x => new UserDataMethodParameter(x)).ToArray();
            }

            public MethodInfo Method { get; }
            public UserDataMethodParameter[] Parameters { get; }
        }

        private struct UserDataMethodParameter
        {
            public UserDataMethodParameter(ParameterInfo info)
            {
                Type = info.ParameterType;
                IsOptional = info.IsOptional;
            }

            public System.Type Type { get; }
            public bool IsOptional { get; }
        }

        public string Name { get; }
        private List<UserDataMethodPart> MethodParts { get; }

        public UserDataMethod(MethodInfo method)
        {
            Name = method.Name;
            var part = new UserDataMethodPart(method);
            MethodParts = new List<UserDataMethodPart>()
            {
                part
            };
        }

        public void LoadMethodPart(MethodInfo method)
        {
            var part = new UserDataMethodPart(method);
            MethodParts.Add(part);
        }

        public MethodInfo GetMethod(System.Type[] parameterTypes)
        {
            foreach (var userDataMethodPart in MethodParts)
            {
                bool valid = true;
                for (var index = 0; index < userDataMethodPart.Parameters.Length; index++)
                {
                    var userDataMethodParameter = userDataMethodPart.Parameters[index];
                    if (index >= parameterTypes.Length)
                    {
                        if (userDataMethodParameter.IsOptional)
                            return userDataMethodPart.Method;
                        valid = false;
                        break;
                    }
                    var parameter = parameterTypes[index];
                    if (userDataMethodParameter.Type != parameter &&
                        !userDataMethodParameter.Type.IsAssignableFrom(parameter))
                    {
                        valid = false;
                        break;
                    }
                }

                if (valid) return userDataMethodPart.Method;
            }

            return null;
        }
    }
}