using System.Collections.Immutable;
using Upsilon.BaseTypes;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public class BoundFunctionCallExpression : BoundExpression
    {
        public BoundExpression Identifier { get; }
        public ImmutableArray<BoundExpression> Parameters { get; }

        public BoundFunctionCallExpression(BoundExpression identifier, ImmutableArray<BoundExpression> parameters,
            TextSpan span) : base(span)
        {
            Identifier = identifier;
            Parameters = parameters;
        }

        public override BoundKind Kind => BoundKind.BoundFunctionCallExpression;
        public override BoundNode GetNodeAtPosition(int characterPosition)
        {
            if (characterPosition >= Identifier.Span.Start && characterPosition <= Identifier.Span.End)
                return Identifier.GetNodeAtPosition(characterPosition);
            foreach (var parameter in Parameters)
            {
                if (characterPosition >= parameter.Span.Start && characterPosition <= parameter.Span.End)
                    return parameter.GetNodeAtPosition(characterPosition);
            }
            return this;

        }

        public override Type Type => Type.Unknown;
    }
}