using System.Collections.Immutable;
using Upsilon.BaseTypes;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public class BoundFunctionExpression : BoundExpression
    {
        public ImmutableArray<BoundVariableSymbol> Parameters { get; }
        public BoundBlockStatement Block { get; set; }

        public BoundFunctionExpression(ImmutableArray<BoundVariableSymbol> parameters, BoundBlockStatement block, TextSpan span) : base(span)
        {
            Parameters = parameters;
            Block = block;
        }

        public override BoundKind Kind => BoundKind.BoundFunctionExpression;
        public override BoundNode GetNodeAtPosition(int characterPosition)
        {
            foreach (var parameter in Parameters)
            {
                if (characterPosition >= parameter.Span.Start && characterPosition <= parameter.Span.End)
                    return parameter.GetNodeAtPosition(characterPosition);
            }
            if (characterPosition >= Block.Span.Start && characterPosition <= Block.Span.End)
                return Block.GetNodeAtPosition(characterPosition);
            return this;
        }

        public override Type Type => Type.Function;
    }
}