using Upsilon.BaseTypes;
using Upsilon.Parser;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public class BoundIndexExpression : BoundExpression
    {
        public BoundIndexExpression(BoundExpression identifier, BoundExpression index, Type type, TextSpan span) : base(span)
        {
            Identifier = identifier;
            Index = index;
            Type = type;
        }

        public BoundExpression Identifier { get; }
        public BoundExpression Index { get; }

        public override BoundKind Kind => BoundKind.BoundIndexExpression;
        public override BoundNode GetNodeAtPosition(int characterPosition)
        {
            if (characterPosition >= Identifier.Span.Start && characterPosition <= Identifier.Span.End)
                return Identifier.GetNodeAtPosition(characterPosition);
            if (characterPosition >= Index.Span.Start && characterPosition <= Index.Span.End)
                return Index.GetNodeAtPosition(characterPosition);
            return this;

        }

        public override Type Type { get; }
    }

    public class BoundFullStopIndexExpression : BoundExpression
    {
        public BoundExpression Expression { get; }
        public string Index { get; }

        public BoundFullStopIndexExpression(BoundExpression expression, string index, TextSpan span) : base(span)
        {
            Expression = expression;
            Index = index;
        }

        public override BoundKind Kind => BoundKind.BoundFullstopIndexExpression;
        public override BoundNode GetNodeAtPosition(int characterPosition)
        {
            if (characterPosition >= Expression.Span.Start && characterPosition <= Expression.Span.End)
                return Expression.GetNodeAtPosition(characterPosition);
            return this;
        }

        public override Type Type => Type.Unknown;
    }
}