using System;
using System.Collections.Generic;
using Upsilon.BaseTypes;
using Upsilon.Binder;

namespace Upsilon.Evaluator
{
    public class EvaluationScope : IDisposable
    {
        private EvaluationScope _parentScope;
        private EvaluationScope _getOnlyParentScope;

        public readonly Dictionary<string, ScriptType> Variables;


        internal EvaluationScope(EvaluationScope parentScope)
        {
            _parentScope = parentScope;
            Variables = new Dictionary<string, ScriptType>();
        }

        internal EvaluationScope(Dictionary<string, ScriptType> vars)
        {
            Variables  = vars;
        }

        internal static EvaluationScope CreateWithGetOnlyParent(EvaluationScope parent)
        {
            var scope = new EvaluationScope(new Dictionary<string, ScriptType>()) {_getOnlyParentScope = parent};
            return scope;
        }

        public void Dispose()
        {
            _parentScope?.Dispose();
            _parentScope = null;
            Variables.Clear();
        }

        public void AssignToNearest(VariableSymbol symbol, ScriptType value)
        {
            if (Variables.ContainsKey(symbol.Name))
            {
                Variables[symbol.Name] = value;
            }
            else if (_parentScope != null)
            {
                _parentScope.AssignToNearest(symbol, value);
            }
            else
            {
                Variables.Add(symbol.Name, value);
            }
        }

        public void CreateLocal(VariableSymbol symbol, ScriptType value)
        {
            Variables[symbol.Name] = value;
        }


        public bool TryGet(VariableSymbol symbol, out ScriptType obj)
        {
            if (Variables.TryGetValue(symbol.Name, out obj))
                return true;
            if (_parentScope != null)
                if (_parentScope.TryGet(symbol, out obj))
                    return true;
            if (_getOnlyParentScope != null)
                if (_getOnlyParentScope.TryGet(symbol, out obj))
                    return true;
            return false;
        }

        public bool TryGet(string variable, out ScriptType obj)
        {
            if (Variables.TryGetValue(variable, out obj))
                return true;
            if (_parentScope != null)
                if (_parentScope.TryGet(variable, out obj))
                    return true;
            if (_getOnlyParentScope != null)
                if (_getOnlyParentScope.TryGet(variable, out obj))
                    return true;
            return false;
        }
    }
}