using System;
using Upsilon.BaseTypes;
using Upsilon.BaseTypes.Number;
using Upsilon.BaseTypes.ScriptTypeInterfaces;

// ReSharper disable UnusedMember.Global
// ReSharper disable MemberCanBePrivate.Global

namespace Upsilon.StandardLibraries
{
    internal class BasicFunctions : LuaLibrary
    {
        [StandardLibraryScriptFunction("assert")]
        public void Assert(ScriptBoolean boolean, ScriptString message = null)
        {
            if (!boolean)
            {
                Error(message ?? new ScriptString("assertion failed!"));
            }
        }

        [StandardLibraryScriptFunction("error")]
        public void Error(ScriptString message)
        {
            throw new Exception(message.Value);
        }

        [StandardLibraryScriptFunction("ipairs")]
        public IIterable UpTillNullPairs(IIterable table)
        {
            return new UpTillNullPairsScriptIterator(table);
        }

        [StandardLibraryScriptFunction("pairs")]
        public IIterable Pairs(IIterable table)
        {
            return new PairsScriptIterator(table);
        }

        [StandardLibraryScriptFunction("tonumber")]
        public ScriptNumber ToNumber(ScriptString obj)
        {
            var str = obj.Value;
            if (str.Contains("."))
            {
                return new ScriptNumberDouble(double.Parse(str));
            }
            else
            {
                return new ScriptNumberLong(long.Parse(str));
            }
        }

        [StandardLibraryScriptFunction("tostring")]
        public ScriptString ToString(ScriptType obj)
        {
            return new ScriptString(obj.ToString());
        }

        [StandardLibraryScriptFunction("type")]
        public ScriptString Type(ScriptType obj)
        {
            return new ScriptString(obj.Type.ToString());
        }

    }
}